unit CPCTB2KImp;

{
Toolbar2000 Import Handler classes
Copyright  2000 by ChoosePill components, Sebastian Modersohn.
Toolbar2000  by Jordan Russell, www.jrsoftware.org
All rights reserved

Use and/or distribution of these classes requires acceptance
of the following Licence Agreement.

LICENSE AGREEMENT:

"Author" herein refers to Sebastian Modersohn from ChoosePill components.
"Software" refers to all files bearing this notice as well as
any other files included with the Toolbar2000 Import Handler classes and
all content in them, regardless of whether any modifications have been made.

1) This notice may not be removed from or altered in any distribution.

2) The Software may NOT be redistributed in ANY FORM (including but
not limited to .PAS, .DCU, .OBJ) after you have obtained it (typically, by
downloading from the Internet) without the express written permission of the
Author.

3) You are permitted to compile the Software into any kind of applications
or Runtime-only packages. ("Compile" refers to the automatic process of
translating the Software's source code into executable Intel x86 machine code
by the Borland Delphi or C++Builder compilers.)

4) This software is provided "as-is", without any express or implied warranty.
In no event shall the Author be held liable for any damages arising from the use
of this software.

5) Full backward compatibility in future versions of the Software is not
guaranteed. In no event shall the Author be held liable for any inconvenience
or damages arising from lack of backward compatibility.

6) The Author may terminate your rights to use the Software for any reason at
his sole discretion.

7) If you do not agree to any of these terms, you are not permitted to use the
Software in any way, and all copies of it must be deleted from your system(s).
}

interface

{$I TB2Ver.inc}

uses
  Forms, Classes, Controls, Windows, TB2Toolbar, TB2Item,
{$IFDEF JR_D6}
  DesignIntf, DesignEditors
{$ELSE}
  DsgnIntf
{$ENDIF}
  {$IFDEF JR_D4} , ImgList {$ENDIF};

type
  TCPCTB2KImportHandler = class(TObject)
  public
    Destination: TComponent;
    Designer: IDesigner;
    procedure Execute(Source: TComponent); virtual;
    function AddItem(ParentItem: TTBCustomItem; ItemClass: TTBCustomItemClass): TTBCustomItem;
    procedure SetImages(AImgList: TCustomImageList);
    function GetImages: TCustomImageList;

    destructor Destroy; override; 
  end;

  TCPCTB2KImportHandlerClass = class of TCPCTB2KImportHandler;

  procedure RegisterTB2KImportHandler(ComponentClass: array of TComponentClass;
    AImportHandler: TCPCTB2KImportHandlerClass);
  function CanImportFrom(AComponent: TComponent): Boolean;
  function GetImportHandler(AComponent: TComponent): TCPCTB2KImportHandlerClass;

implementation

var ImpList: TStringList;

type
  TCPCClassDesc = class(TObject)
  public
    TheClass: TClass;
    ImportHandler: TCPCTB2KImportHandlerClass;
  end;

function CanImportFrom(AComponent: TComponent): Boolean;
var i: Integer;
    CD: TCPCClassDesc;
begin
  //this is a quick check using the is operator; in GetImportHandler (below) the search is more
  //appropriate as it scans all ancestors until it finds an exact matching entry in ImpList
  Result:=False;
  for i:=0 to ImpList.Count-1 do
  begin
    CD:=TCPCClassDesc(ImpList.Objects[i]);
    Result:=AComponent is CD.TheClass;
    if Result then
      Break;
  end;
end;

function GetImportHandler(AComponent: TComponent): TCPCTB2KImportHandlerClass;
var TheClass: TClass;

  function GetHandlerOfClass(AClass: TClass): TCPCTB2KImportHandlerClass;
  var i: Integer;
      CD: TCPCClassDesc;
  begin
    Result:=nil;
    for i:=0 to ImpList.Count-1 do
    begin
      CD:=TCPCClassDesc(ImpList.Objects[i]);
      if CD.TheClass=AClass then
      begin
        Result:=CD.ImportHandler;
        Break;
      end;
    end;
  end;

begin
  TheClass:=AComponent.ClassType;
  repeat
    Result:=GetHandlerOfClass(TheClass);
    TheClass:=TheClass.ClassParent;
  until Assigned(Result) or (TheClass=nil) or (TheClass=TComponent);
end;

procedure RegisterTB2KImportHandler(ComponentClass: array of TComponentClass;
  AImportHandler: TCPCTB2KImportHandlerClass);
var i: Integer;
    ClassDesc: TCPCClassDesc;
begin
  for i:=0 to High(ComponentClass) do
  begin
    ClassDesc:=TCPCClassDesc.Create;
    try
      ClassDesc.TheClass:=ComponentClass[i];
      ClassDesc.ImportHandler:=AImportHandler;

      ImpList.AddObject(ClassDesc.TheClass.ClassName, ClassDesc);
    except
      ClassDesc.Free;
    end;
  end;
end;

{ TCPCTB2KImportHandler }

function TCPCTB2KImportHandler.AddItem(ParentItem: TTBCustomItem;
  ItemClass: TTBCustomItemClass): TTBCustomItem;
begin
  Result:=nil;
  if Assigned(Destination) and Assigned(ItemClass) then
  begin
    if not Assigned(ParentItem) then
    begin
      if Destination is TTBCustomToolbar then
        ParentItem:=TTBCustomToolbar(Destination).Items;
      if Destination is TTBItemContainer then
        ParentItem:=TTBItemContainer(Destination).Items;
      if Destination is TTBPopupMenu then
        ParentItem:=TTBPopupMenu(Destination).Items;
      if not Assigned(ParentItem) then
        Exit;
    end;
    Result:=ItemClass.Create(Destination.Owner);
    try
      if Assigned(Designer) then
        Result.Name:=Designer.UniqueName(ItemClass.ClassName);
      ParentItem.Add(Result);
    except
      Result.Free;
      raise;
    end;
  end;
end;

destructor TCPCTB2KImportHandler.Destroy;
begin
  Designer:=nil;
  inherited;
end;

procedure TCPCTB2KImportHandler.Execute(Source: TComponent);
begin
//do nothing; descendant have to override this...
end;

function TCPCTB2KImportHandler.GetImages: TCustomImageList;
begin
  Result:=nil;
  if Destination is TTBCustomToolbar then
    Result:=TTBCustomToolbar(Destination).Images;
  if Destination is TTBItemContainer then
    Result:=TTBItemContainer(Destination).Images;
  if Destination is TTBPopupMenu then
    Result:=TTBPopupMenu(Destination).Images;
end;

procedure TCPCTB2KImportHandler.SetImages(AImgList: TCustomImageList);
begin
  if Destination is TTBCustomToolbar then
  begin
    if TTBCustomToolbar(Destination).Images=nil then
      TTBCustomToolbar(Destination).Images:=AImgList;
  end;
  if Destination is TTBItemContainer then
  begin
    if TTBItemContainer(Destination).Images=nil then
      TTBItemContainer(Destination).Images:=AImgList;
  end;
  if Destination is TTBPopupMenu then
  begin
    if TTBPopupMenu(Destination).Images=nil then
      TTBPopupMenu(Destination).Images:=AImgList;
  end;
end;

initialization
  ImpList:=TStringList.Create;
  ImpList.Sorted:=True;
  ImpList.Duplicates:=dupAccept;
finalization
  while ImpList.Count>0 do
  begin
    TObject(ImpList.Objects[0]).Free;
    ImpList.Delete(0);
  end;
  ImpList.Free;
end.
