unit CPCTB2KImpMenus;

{
Toolbar2000 Import Handler classes
Copyright  2000 by ChoosePill components, Sebastian Modersohn.
Toolbar2000  by Jordan Russell, www.jrsoftware.org
All rights reserved

Use and/or distribution of these classes requires acceptance
of the following Licence Agreement.

LICENSE AGREEMENT:

"Author" herein refers to Sebastian Modersohn from ChoosePill components.
"Software" refers to all files bearing this notice as well as
any other files included with the Toolbar2000 Import Handler classes and
all content in them, regardless of whether any modifications have been made.

1) This notice may not be removed from or altered in any distribution.

2) The Software may NOT be redistributed in ANY FORM (including but
not limited to .PAS, .DCU, .OBJ) after you have obtained it (typically, by
downloading from the Internet) without the express written permission of the
Author.

3) You are permitted to compile the Software into any kind of applications
or Runtime-only packages. ("Compile" refers to the automatic process of
translating the Software's source code into executable Intel x86 machine code
by the Borland Delphi or C++Builder compilers.)

4) This software is provided "as-is", without any express or implied warranty.
In no event shall the Author be held liable for any damages arising from the use
of this software.

5) Full backward compatibility in future versions of the Software is not
guaranteed. In no event shall the Author be held liable for any inconvenience
or damages arising from lack of backward compatibility.

6) The Author may terminate your rights to use the Software for any reason at
his sole discretion.

7) If you do not agree to any of these terms, you are not permitted to use the
Software in any way, and all copies of it must be deleted from your system(s).
}

{ FILE DESCRIPTION:

this unit implements a class which descends from TCPCTB2KImportHandler and is
capable of importing TMenuItems, TMainMenus and TPopupMenus into any of the
Toolbar2000 controls which can have items. This is done in the overriden Execute
method of TCPCMenuImportHandler. The Destination property holds a reference to the
component which should retrieve the items.

Note that TCPCTB2KImportHandler provides methods to simplify item creation.
Use AddItem to add an item to the Destination (which may be a TTBCustomToolbar,
a TTBPopupMenu or a TTBItemContainer). Use SetImages to set the Images property of
the Destination (works only if it's not already assigned).

In addition note that this unit contains a Register procedure which is called when
this unit is added to a design time package. It calls the method

  RegisterTB2KImportHandler

to register the ImportHandler class for components of type TMainMenu, TMenuItem and TPopupMenu!
}

interface

{$I TB2Ver.inc}

uses Windows, Classes, Forms, Menus, CPCTB2KImp, TB2Toolbar, TB2Item, TB2Dock
     {$IFDEF JR_D4} , ImgList {$ENDIF};

type
  TCPCMenuImportHandler = class(TCPCTB2KImportHandler)
  public
    procedure Execute(Source: TComponent); override;
  end;

procedure Register;

implementation

procedure Register;
begin
  //register new Import Handler class; this is necessary to see components of class
  //TMainMenu, TPopupMenu or TMenuItem in the "Choose Component to import from" dialog.
  RegisterTB2KImportHandler([TMainMenu, TPopupMenu, TMenuItem], TCPCMenuImportHandler);
end;

{ TCPCMenuImportHandler }

procedure TCPCMenuImportHandler.Execute(Source: TComponent);

  //this method is called recursively for all MenuItems;
  //if the IgnoreRoot parameter is True, no new item is created, but the method
  //is called for every child menuitem (this is necessary the calling AddMenuItem
  //with TMainMenu.Items etc)
  procedure AddMenuItem(AItem: TMenuItem; Parent: TTBCustomItem; IgnoreRoot: Boolean);
  var NewParent: TTBCustomItem;
      i: Integer;
  begin
    if Assigned(AItem) then
    begin
      if not IgnoreRoot then
      begin
        {$IFDEF JR_D5}
        if AItem.IsLine then
        {$ELSE}
        if AItem.Caption = '-' then
        {$ENDIF}
          NewParent:=AddItem(Parent, TTBSeparatorItem) else
        begin
          if AItem.Count>0 then
            NewParent:=AddItem(Parent, TTBSubMenuItem) else
            NewParent:=AddItem(Parent, TTBItem);

          if Assigned(NewParent) then
          begin
            NewParent.Action:=AItem.Action;

            NewParent.Caption:=AItem.Caption;
            NewParent.Hint:=AItem.Hint;
            NewParent.Checked:=AItem.Checked;
  //          NewParent.Default:=AItem.Default;
            NewParent.Enabled:=AItem.Enabled;
  //          NewParent.GroupIndex:=AItem.GroupIndex;
            NewParent.HelpContext:=AItem.HelpContext;
            NewParent.ImageIndex:=AItem.ImageIndex;
  //          NewParent.RadioItem:=AItem.RadioItem;
            NewParent.Shortcut:=AItem.Shortcut;
            NewParent.Tag:=AItem.Tag;
            NewParent.Visible:=AItem.Visible;
            if NewParent is TTBItem then
              TTBItem(NewParent).OnClick:=AItem.OnClick;
          end;
        end;

      end else
        NewParent:=Parent;
      for i:=0 to AItem.Count-1 do
        AddMenuItem(AItem.Items[i], NewParent, False);
    end;
  end;

var ParentItem: TMenuItem;
    Images: TCustomImageList;
    ParentMenu: TMenu;
begin
//Destination holds the component to which the new items will be added
//it isn't used directly here, but the AddItem method uses it
  if Assigned(Destination) and (Destination<>Source) then
  begin
    //if Source is a TMenu (i.e. a TMainMenu or a TPopupMenu or a descendant)
    //just call AddMenuItem with the Root TMenuItem and save its ImageList
    Images:=nil;
    if Source is TMenu then
    begin
      AddMenuItem(TMenu(Source).Items, nil, True);
      Images:=TMenu(Source).Images;
      if (Source is TMainMenu) and (Destination is TTBCustomToolbar) then
        TTBCustomToolbar(Destination).MenuBar:=True;
    end else
    begin
      //if Source is a TMenuItem, call AddMenuItem with the Root TMenuItem
      //to get its correct ImageList walk through the Parents and Check the
      //SubMenuImages property or at least the Images property of the parent TMenu
      if Source is TMenuItem then
      begin
        AddMenuItem(TMenuItem(Source), nil, False);
        {$IFDEF JR_D5}
        ParentItem:=TMenuItem(Source).Parent;
        while Assigned(ParentItem) and (not Assigned(Images)) do
        begin
          Images:=ParentItem.SubmenuImages;
          ParentItem:=ParentItem.Parent
        end;
        {$ENDIF}
        if not Assigned(Images) then
        begin
          ParentMenu:=TMenuItem(Source).GetParentMenu;
          if Assigned(ParentMenu) then
            Images:=ParentMenu.Images;
        end;
      end;
    end;
    //set the Images property of the Destination appropriately
    //SetImages works only if Destination.Images is nil
    if Assigned(Images) then
      SetImages(Images);
  end;
end;

end.
 